@@ -47,3 +47,7 @@ from pybsc.stdout_utils import suppress_stdout
 
 from pybsc.str_utils import removesuffix
 from pybsc.str_utils import remove_non_ascii
+
+from pybsc.bytes_utils import bytes_to_uint8_list
+
+from pybsc.subprocess_utils import run_command
