@@ -2,6 +2,8 @@ from __future__ import division
 
 import base64
 from enum import Enum
+import math
+from pathlib import Path
 
 import cv2
 import numpy as np
@@ -75,6 +77,68 @@ def pil_to_cv2_interpolation(interpolation):
     return cv_interpolation
 
 
+def convert_to_numpy(input_data):
+    """Convert the input data into a numpy.ndarray object.
+
+    Parameters
+    ----------
+    input_data : str, Path, np.ndarray, or PIL.Image
+        The input data to be converted. This can be a string or Path
+        object representing a file path, a np.ndarray representing an image,
+        or a PIL.Image object.
+
+    Returns
+    -------
+    np.ndarray
+        The converted np.ndarray object.
+
+    Raises
+    ------
+    TypeError
+        If the input_data is not of type str, Path, np.ndarray, or PIL.Image.
+    """
+    if isinstance(input_data, (str, Path)):
+        return np.array(PIL.Image.open(str(input_data)))
+    elif isinstance(input_data, np.ndarray):
+        return input_data
+    elif isinstance(input_data, PILImage.Image):
+        return np.array(input_data)
+    else:
+        raise TypeError(
+            "Invalid input type. Expected filepath, np.ndarray, or PIL.Image")
+
+
+def convert_to_pil(input_data):
+    """Convert the input data into a PIL.Image object.
+
+    Parameters
+    ----------
+    input_data : str, Path, np.ndarray, or PIL.Image
+        The input data to be converted.
+        This can be a string or Path object representing a file path,
+        a np.ndarray representing an image, or a PIL.Image object.
+
+    Returns
+    -------
+    PIL.Image
+        The converted PIL.Image object.
+
+    Raises
+    ------
+    TypeError
+        If the input_data is not of type str, Path, np.ndarray, or PIL.Image.
+    """
+    if isinstance(input_data, (str, Path)):
+        return PIL.Image.open(str(input_data))
+    elif isinstance(input_data, np.ndarray):
+        return PIL.Image.fromarray(input_data)
+    elif isinstance(input_data, PIL.Image):
+        return input_data
+    else:
+        raise TypeError(
+            "Invalid input type. Expected filepath, np.ndarray, or PIL.Image")
+
+
 def decode_image_cv2(b64encoded):
     bin = b64encoded.split(",")[-1]
     bin = base64.b64decode(bin)
@@ -214,6 +278,33 @@ def resize_keeping_aspect_ratio_wrt_longside(img, length,
         return resized_img
 
 
+def _resize_keeping_aspect_ratio_wrt_longside_rgba(img, image_width):
+    img = convert_to_numpy(img)
+    try:
+        img, mask = img[..., :3], img[..., 3]
+    except IndexError:
+        height = img.shape[0]
+        width = img.shape[1]
+        mask = 255 * np.ones(
+            (height, width),
+            dtype=np.uint8)
+    pil_mask = PIL.Image.fromarray(mask)
+    pil_mask = resize_keeping_aspect_ratio_wrt_longside(
+        pil_mask, image_width,
+        interpolation='nearest')
+    pil_img = PIL.Image.fromarray(img)
+    pil_img = resize_keeping_aspect_ratio_wrt_longside(
+        pil_img, image_width,
+        interpolation='bilinear')
+    return np.concatenate(
+        [np.array(pil_img, dtype=np.uint8),
+         np.array(pil_mask, dtype=np.uint8).reshape(
+             pil_mask.size[1],
+             pil_mask.size[0],
+             1)],
+        axis=2)
+
+
 def resize_keeping_aspect_ratio_wrt_target_size(
         img, width, height, interpolation='bilinear',
         background_color=(0, 0, 0)):
@@ -232,12 +323,14 @@ def resize_keeping_aspect_ratio_wrt_target_size(
         background_color)
 
 
-def squared_padding_image(img, length=None):
+def squared_padding_image(img, length=None,
+                          return_offset=None):
     H, W = img.shape[:2]
     if H > W:
         if length is not None:
             img = resize_keeping_aspect_ratio_wrt_longside(img, length)
         margin = img.shape[0] - img.shape[1]
+        offset = (margin // 2, 0)
         img = np.pad(img,
                      [(0, 0),
                       (margin // 2, margin - margin // 2),
@@ -246,10 +339,14 @@ def squared_padding_image(img, length=None):
         if length is not None:
             img = resize_keeping_aspect_ratio_wrt_longside(img, length)
         margin = img.shape[1] - img.shape[0]
+        offset = (0, margin // 2)
         img = np.pad(img,
                      [(margin // 2, margin - margin // 2),
                       (0, 0), (0, 0)], 'constant')
-    return img
+    if return_offset:
+        return img, offset
+    else:
+        return img
 
 
 def concat_with_keeping_aspect(
@@ -407,6 +504,55 @@ def tile_image(wh_size, tile_size_wh, window_size=None):
          for i in range(len(sx))])
 
 
+def create_tile_image(image_list,
+                      image_size=300,
+                      num_tiles_per_row=10,
+                      background_color=(255, 255, 255)):
+    """Create a tiled image from a list of images.
+
+    Parameters
+    ----------
+    image_list : list
+        A list of images to be tiled. Each image can be a filepath
+        (str or Path), np.ndarray, or PIL.Image.
+    image_size : int
+        The size for each individual image tile. Defaults to 300.
+    num_tiles_per_row : int, optional
+        The number of image tiles per row in the output image.
+        Defaults to 10.
+    background_color : tuple, optional
+        The background color for the output image in (R, G, B).
+        Defaults to (255, 255, 255).
+
+    Returns
+    -------
+    PIL.Image
+        The created tiled image.
+
+    Raises
+    ------
+    ValueError
+        If any of the images in image_list cannot be opened or read.
+    """
+    num_images = len(image_list)
+    num_rows = math.ceil(num_images / num_tiles_per_row)
+
+    tile_width = image_size * num_tiles_per_row
+    tile_height = image_size * num_rows
+    tile_image = PIL.Image.new(
+        'RGBA', (tile_width, tile_height),
+        tuple(list(background_color) + [0]))
+
+    for i, image in enumerate(image_list):
+        tile_x = (i % num_tiles_per_row) * image_size
+        tile_y = (i // num_tiles_per_row) * image_size
+        pil_img = convert_to_pil(
+            _resize_keeping_aspect_ratio_wrt_longside_rgba(
+                image, image_size))
+        tile_image.paste(pil_img, (tile_x, tile_y))
+    return tile_image
+
+
 def get_bboxes_from_tile(bboxes, tile):
     new_bboxes = []
     for y1, x1, y2, x2 in bboxes:
@@ -462,3 +608,81 @@ def non_maximum_suppression(bbox, thresh, score=None, limit=None):
     if score is not None:
         indices = order[indices]
     return indices.astype(np.int32)
+
+
+def rotate(pil_img, mask=None, angle=360):
+    return_np = False
+    concat = False
+    if isinstance(pil_img, np.ndarray):
+        if pil_img.shape[2] == 4:
+            concat = True
+            pil_img, mask = PIL.Image.fromarray(pil_img[..., :3]), \
+                PIL.Image.fromarray(pil_img[..., 3])
+        else:
+            pil_img = PIL.Image.fromarray(pil_img)
+            if mask is not None:
+                mask = PIL.Image.fromarray(mask)
+        return_np = True
+
+    rot_img = pil_img.rotate(angle, resample=PIL.Image.BILINEAR,
+                             expand=True)
+    if mask is not None:
+        if return_np:
+            if concat:
+                rgb = np.array(rot_img)
+                mask = np.array(mask.rotate(
+                    angle,
+                    resample=PIL.Image.NEAREST,
+                    expand=True), dtype=np.uint8)
+                return np.concatenate(
+                    [rgb, mask[..., None]], axis=2)
+            return np.array(rot_img), np.array(mask.rotate(
+                angle,
+                resample=PIL.Image.NEAREST,
+                expand=True), dtype=np.uint8)
+        else:
+            return rot_img, mask.rotate(
+                angle,
+                resample=PIL.Image.NEAREST,
+                expand=True)
+    if return_np:
+        return np.array(rot_img)
+    return rot_img
+
+
+def rescale(pil_img, mask=None, scale=1.0):
+    if isinstance(pil_img, np.ndarray):
+        concat = False
+        if pil_img.shape[2] == 4:
+            concat = True
+            pil_img, mask = PIL.Image.fromarray(
+                pil_img[..., :3]), \
+                PIL.Image.fromarray(pil_img[..., 3])
+        else:
+            pil_img = PIL.Image.fromarray(pil_img)
+            if mask is not None:
+                mask = PIL.Image.fromarray(mask)
+        w, h = pil_img.size
+        w = int(np.ceil(scale * pil_img.size[0]))
+        h = int(np.ceil(scale * pil_img.size[1]))
+        pil_img = pil_img.resize((w, h), PIL.Image.BILINEAR)
+        rgb = np.array(pil_img, dtype=np.uint8)
+        if mask is not None:
+            mask = mask.resize((w, h), PIL.Image.NEAREST)
+            mask = np.array(mask, dtype=np.uint8)
+            if concat:
+                np.concatenate(
+                    [rgb, mask[..., None]], axis=2)
+            else:
+                return rgb, mask
+        return rgb
+
+    else:
+        w, h = pil_img.size
+        w = int(np.ceil(scale * pil_img.size[0]))
+        h = int(np.ceil(scale * pil_img.size[1]))
+        pil_img = pil_img.resize((w, h), PIL.Image.BILINEAR)
+        if mask is not None:
+            mask = mask.resize((w, h), PIL.Image.NEAREST)
+            return pil_img, mask
+        return pil_img
