@@ -16,6 +16,7 @@ import cv2
 import skvideo.io
 from tqdm import tqdm
 
+from pybsc.subprocess_utils import run_command
 from pybsc.timestamp_utils import get_timestamp_indices_wrt_base
 
 
@@ -375,3 +376,18 @@ def split_video_to_fileframe_from_sections(
             shutil.copy(filename, frame_outpath / filename_tmpl.format(i))
         output_frame_parent_paths.append(frame_outpath)
     return output_frame_parent_paths
+
+
+def get_video_rotation(video_file_path):
+    """Function to get the rotation of the input video file.
+
+    Returns a rotation None, 90, 180 or 270
+    """
+    cmd = "ffprobe -loglevel error -select_streams v:0 -show_entries stream_tags=rotate -of default=nw=1:nk=1 {}".format(video_file_path)  # NOQA
+    ffprobe_output = run_command(cmd, shell=True, capture_output=True)
+    # Output of cmdis None if it should be 0
+    if len(ffprobe_output.stdout) > 0:
+        rotation = int(ffprobe_output.stdout)
+    else:
+        rotation = 0
+    return rotation
